-------------------------------------------------------------------------------
--! @file       CipherCore.vhd
--! @author     Farnoud Farahmand
--! @brief      CLOC CipherCore
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.ALL;

entity CipherCore is
    generic (
        --! Reset behavior
        G_ASYNC_RSTN    : boolean := False; --! Async active low reset
        --! Block size (bits)
        G_DBLK_SIZE     : integer := 128;   --! Data
        G_KEY_SIZE      : integer := 128;   --! Key
        G_TAG_SIZE      : integer := 128;   --! Tag
        --! The number of bits required to hold block size expressed in
        --! bytes = log2_ceil(G_DBLK_SIZE/8)
        G_LBS_BYTES     : integer := 4
    );
    port (
        --! Global
        clk             : in  std_logic;
        rst             : in  std_logic;
        --! PreProcessor (data)
        key             : in  std_logic_vector(G_KEY_SIZE       -1 downto 0);
        bdi             : in  std_logic_vector(G_DBLK_SIZE      -1 downto 0);
        --! PreProcessor (controls)
        key_ready       : out std_logic;
        key_valid       : in  std_logic;
        key_update      : in  std_logic;
        decrypt         : in  std_logic;
        bdi_ready       : out std_logic;
        bdi_valid       : in  std_logic;
        bdi_type        : in  std_logic_vector(3                -1 downto 0);
        bdi_partial     : in  std_logic;
        bdi_eot         : in  std_logic;
        bdi_eoi         : in  std_logic;
        bdi_size        : in  std_logic_vector(G_LBS_BYTES+1    -1 downto 0);
        bdi_valid_bytes : in  std_logic_vector(G_DBLK_SIZE/8    -1 downto 0);
        bdi_pad_loc     : in  std_logic_vector(G_DBLK_SIZE/8    -1 downto 0);
        --! PostProcessor
        bdo             : out std_logic_vector(G_DBLK_SIZE      -1 downto 0);
        bdo_valid       : out std_logic;
        bdo_ready       : in  std_logic;
        bdo_size        : out std_logic_vector(G_LBS_BYTES+1    -1 downto 0);
        msg_auth_done   : out std_logic;
        msg_auth_valid  : out std_logic
    );
end entity CipherCore;

architecture structure of CipherCore is

-----------DATAPATH------------------------------------------------	  		  
	    signal  start         :  std_logic;
	    signal  tag_match     :  std_logic;
        signal  done          :  std_logic;
		signal  done_init     :  std_logic;
		signal  len_a_s       :  std_logic;
		signal  len_d_s       :  std_logic;
		signal  msb_a         :  std_logic;
     
------------------------------------------------------------------
        signal init           :  std_logic;		
   	    signal sel_ph1        :  std_logic;
		signal sel_ph2        :  std_logic;
		signal sel_ph3        :  std_logic;
	    signal sel_ph4        :  std_logic;
		signal sel_ph5        :  std_logic;
		signal sel_ph6        :  std_logic_vector (1 downto 0);
		signal sel_ph7        :  std_logic;		
		signal sel_ph_last    :  std_logic_vector(1 downto 0);
		signal sel_ed1        :  std_logic;
		signal sel_bdo        :  std_logic;
		signal init_enc_dec   :  std_logic;
-------------------------------------------------------------------
        signal en_npub_reg      : std_logic;
		
begin
    Datapath: entity work.top_level_datapath(dataflow) 
    port map (
			     rst                   => rst,
		         clk                   => clk,	
                 bdi                   => bdi,
				 key                   => key,
				 bdo                   => bdo,	  
		         start                 => start,				 
		         tag_match             => tag_match,            
                 done                  => done,				
		         done_init             => done_init,				 
				 init                  => init,
		         sel_ph1               => sel_ph1,
		         sel_ph2               => sel_ph2,
				 sel_ph3               => sel_ph3,
		         sel_ph4               => sel_ph4,
		         sel_ph5               => sel_ph5,
				 sel_ph6               => sel_ph6,  
		         sel_ph7               => sel_ph7,
				 sel_ph_last           => sel_ph_last,
		         sel_ed1               => sel_ed1,
				 sel_bdo               => sel_bdo,					
				 init_enc_dec          => init_enc_dec,
		         --len_a_s               => len_a_s,
				 --len_d_s               => len_d_s,				 
                 msb_a                 => msb_a,
				 en_npub_reg		   => en_npub_reg,
			     bdi_valid_bytes	   => bdi_valid_bytes,
			     bdi_pad_loc 		   => bdi_pad_loc
				 );
				 
	   Controller: entity work.cloc_control(behavioral) 
         port map (		 
		             rst                   => rst,
					 clk                   => clk,	  	  
					 start                 => start, 			 
					 tag_match             => tag_match,            
					 done                  => done,				
					 done_init             => done_init,				 
					 init                  => init,
					 sel_ph1               => sel_ph1,
					 sel_ph2               => sel_ph2,
					 sel_ph3               => sel_ph3,
					 sel_ph4               => sel_ph4,
					 sel_ph5               => sel_ph5,
					 sel_ph6               => sel_ph6,  
					 sel_ph7               => sel_ph7,
					 sel_ph_last           => sel_ph_last,
					 sel_ed1               => sel_ed1,
					 sel_bdo               => sel_bdo,				 
					 init_enc_dec          => init_enc_dec,
					 --len_a_s               => len_a_s,
					 --len_d_s               => len_d_s,				 
					 msb_a                 => msb_a,
					 key_update            => key_update,
					 key_valid             => key_valid,
					 key_ready             => key_ready,
					 bdi_valid             => bdi_valid,
					 bdi_ready             => bdi_ready,
					 bdi_decrypt           => decrypt,
					 bdi_eot               => bdi_eot,
					-- bdi_size              => bdi_size,
					 bdo_ready             => bdo_ready,
					 bdo_valid             => bdo_valid,
					 msg_auth_valid        => msg_auth_valid,
					 msg_auth_done         => msg_auth_done,
					 en_npub_reg		   => en_npub_reg,
			         bdi_pad_loc 		   => bdi_pad_loc
				 );

end structure;