-------------------------------------------------------------------------------
--! @file       cloc_controller.vhd
--! @author     Farnoud Farahmand
--! @brief      Control unit for CLOC 
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;

entity cloc_control is
    port(  
        clk       : in std_logic;
        rst       : in std_logic;
			 
        --! Internal
		-------MUX-INIT---------
		sel_ph0 	 : out std_logic;
		sel_ph1 	 : out std_logic;
		sel_ph2 	 : out std_logic;
		sel_ph3 	 : out std_logic;
		sel_ph4 	 : out std_logic;
		sel_ph5 	 : out std_logic;
		sel_ph6 	 : out std_logic_vector (1 downto 0);
		sel_ph7 	 : out std_logic;
		sel_ed1 	 : out std_logic;
		sel_ph_last  : out std_logic_vector (1 downto 0);
		init_enc_dec : out std_logic;		 
		sel_bdo   : out std_logic; 
        --------EN-----------		  
		tag_match    : in    std_logic;
		bdi_pad_loc  : in std_logic_vector(15 downto 0);
		msb_a        : in std_logic;
	    en_npub_reg  : out std_logic;
	    --------AES-----------
		start     : out std_logic;
	    init      : out std_logic;
		done      : in  std_logic;
	    done_init : in  std_logic;
	    --! External
		key_update 		 : in  std_logic;
		key_valid        : in  std_logic;
		key_ready        : out std_logic;
		bdi_valid        : in  std_logic;
		bdi_ready        : out std_logic;
		bdi_decrypt      : in  std_logic;
		bdi_eot          : in  std_logic;
		bdo_valid        : out std_logic;
		bdo_ready        : in  std_logic;
		msg_auth_valid   : out std_logic;
		msg_auth_done    : out std_logic
		);
			
end cloc_control;

architecture behavioral of cloc_control is

		--! Internal Registers
		type t_state is (S_RESET, S_KEY_SCH, S_AD_INIT, S_AD_ENC,
						 S_AD_LASTBLOCK, S_V_CAL, S_ENC_DEC, S_WR,
						 S_TAG_CAL, S_WR_TAG, S_TAG_DEC, S_KEY_READ, S_NPUB_READ ); 
							       
		signal state            : t_state;
        signal state_next       : t_state;
		signal msb_r            : std_logic;
		signal eot_r            : std_logic;
		signal decrypt_r        : std_logic;
		signal lastbl_r         : std_logic;
		signal firstblock_r     : std_logic;
		signal firstblock_next  : std_logic;
        signal secblock_r       : std_logic;
		signal secblock_next    : std_logic;		
		signal enc_dec_r        : std_logic;
		signal enc_dec_next     : std_logic;
		
		--! Internal signals
		signal en_msb_s         : std_logic;
		signal en_eot_s         : std_logic;
		signal en_lastbl_s      : std_logic;
		signal en_decrypt_s     : std_logic;

		
begin
   
	p_fsm: process(clk)
    begin
        if rising_edge(clk) then
			if (rst = '1') then
				state   <= S_RESET;
			else
				state   <= state_next;
			end if;
			
			if (en_msb_s = '1') then
				msb_r <= msb_a;
			end if;
			
			if (en_decrypt_s = '1') then
				decrypt_r <= bdi_decrypt;
			end if;
			
		    if (en_eot_s = '1') then
				eot_r    <= bdi_eot;
			end if;
			
			if (en_lastbl_s = '1') then
				lastbl_r <= bdi_eot;
			end if;
			
			firstblock_r  <= firstblock_next;
			secblock_r    <= secblock_next;
		end if; 
    end process;
		
		
     p_comb: process(state, done, done_init, key_update, msb_r, bdi_pad_loc, 
	                 key_valid, bdi_valid, bdi_decrypt, bdi_eot, eot_r, secblock_r,
					 lastbl_r, bdo_ready, firstblock_r, tag_match )
	    begin
          --! Default values
			state_next   <= state; 
			sel_ph0      <= '1';
			sel_ph1      <= '0';
			sel_ph2      <= '1';
			sel_ph3      <= '1';
			sel_ph4      <= '1';
			sel_ph5      <= '1';
			sel_ph6      <= "10";
			sel_ph7      <= '1';
			sel_ed1      <= '1';
		   sel_bdo	    <= '0';
			sel_ph_last  <= "00";
			start        <= '0';
         init         <= '0';
         init_enc_dec <= '1';
         bdi_ready    <= '0';
			key_ready	 <= '0';
		   bdo_valid    <= '0';
			en_msb_s     <= '0';
    		en_eot_s     <= '0';
			en_lastbl_s  <= '0';
			en_npub_reg	 <= '0';
			en_decrypt_s   <= '0';
			msg_auth_valid <= '0';
			msg_auth_done  <= '0';
			 
		case state is
		        when S_RESET =>
					if (bdi_valid = '1')then
					    if (key_update = '1')then
                    	    state_next <= S_KEY_READ;
                    	else
                        	state_next <= S_NPUB_READ;
						end if; 					  
			        end if;
					
				when S_KEY_READ =>
					key_ready <= '1';
					if (key_valid = '1')then
						init <= '1';
						state_next <= S_KEY_SCH;
			        end if;	
				 
				when S_KEY_SCH =>
				    if (Done_init = '1')then
                    	state_next <= S_NPUB_READ;				  
			        end if;
					
				when S_NPUB_READ =>
					 bdi_ready <= '1';
				    if (bdi_valid = '1')then
						en_npub_reg     <= '1';
						firstblock_next <= '1';
                    	state_next <= S_AD_INIT;				  
			        end if;
				  
				when S_AD_INIT =>
					bdi_ready <= '1';
					if(bdi_valid = '1')then
						if (firstblock_r = '1')then																
									secblock_next <= '1';
									en_msb_s      <= '1';
									sel_ph1       <= '1';
									sel_ph3       <= '0';
									sel_ph6       <= "10";
									start         <= '1';
									init_enc_dec  <= '0';
									en_eot_s      <= '1';
									state_next    <= S_AD_ENC;
													
					    else
									if (secblock_r = '1')then
												if (msb_r = '1')then
															secblock_next <= '0';
															sel_ph4       <= '0';
															sel_ph3       <= '0';
															sel_ph6       <= "10";
															start         <= '1';
															init_enc_dec  <= '0';
															en_eot_s      <= '1';
															state_next    <= S_AD_ENC;
												else
														    secblock_next <= '0';
															sel_ph3       <= '0';
															sel_ph6       <= "10";
															start         <= '1';
															init_enc_dec  <= '0';
															en_eot_s      <= '1';
															state_next    <= S_AD_ENC;
												end if; 	 
									else
											   sel_ph3      <= '0';
												sel_ph6      <= "10";
												start        <= '1';
												init_enc_dec <= '0';
												en_eot_s     <= '1';
												state_next   <= S_AD_ENC;
									end if;			 											
						end if;															  
					end if;		 
				
				when S_AD_ENC =>
					if (done = '1')then
						if(eot_r = '1')then
							state_next <= S_AD_LASTBLOCK;
						else
							firstblock_next <= '0';
							state_next 		<= S_AD_INIT;
					    end if;		
					end if; 
					   
				when S_AD_LASTBLOCK =>
					Sel_ph5    <= '0';
					en_eot_s   <= '1';
					if (firstblock_r = '1')then
						if (msb_r = '1')then							
							sel_ph4      <= '0';
							if (bdi_pad_loc /= x"0000")then
								Sel_ph6      <= "01";
								Sel_ph5      <= '0';
								Sel_ed1      <= '0';
								Start        <= '1';
							    state_next <= S_V_CAL;
							else
							    Sel_ph6      <= "00";
								Sel_ph5      <= '0';
								Sel_ed1      <= '0';
								Start        <= '1';
							    state_next <= S_V_CAL;
							end if;												 
					    else	
							if (bdi_pad_loc /= x"0000")then
								Sel_ph6      <= "01";
								Sel_ph5      <= '0';
								Sel_ed1      <= '0';
								Start        <= '1';
							    state_next <= S_V_CAL;												 
							else
							    Sel_ph6      <= "00";
								Sel_ph5      <= '0';
								Sel_ed1      <= '0';
								Start        <= '1';
							    state_next <= S_V_CAL;
							end if;
						end if;
					else
					    if (bdi_pad_loc /= x"0000")then
							Sel_ph6      <= "01";
							Sel_ph5      <= '0';
							Sel_ed1      <= '0';
							Start        <= '1';
							state_next <= S_V_CAL;
					    else
							Sel_ph6      <= "00";
							Sel_ph5      <= '0';
							Sel_ed1      <= '0';
							Start        <= '1';
							state_next <= S_V_CAL;
						end if;  							
				    end if;
                   
				when S_V_CAL =>
					en_decrypt_s   <= '1';
				    if (done = '1')then
						state_next <= S_ENC_DEC;									  
					end if;
								 						 
			    when S_ENC_DEC =>
					bdi_ready <= '1';
					--en_decrypt_s   <= '1';
					if (bdi_valid = '1')then
								if (bdi_eot = '1')then 
									    en_eot_s     <= '1';      
									    if (bdi_pad_loc /= x"0000")then
												sel_ph_last  <= "10";
												if (bdo_ready = '1')then
														bdo_valid     <= '1';
														sel_ph3        <= '0';
														if (bdi_decrypt = '1')then
																init_enc_dec <= '0';
																start        <= '1';
																state_next   <= S_TAG_CAL;
														else
																start        <= '1';
																state_next   <= S_TAG_CAL;																					
														end if;
												else
														state_next   <= S_WR;
												end if;	
										else
												sel_ph_last  <= "01";
												if (bdo_ready = '1')then
														bdo_valid     <= '1';
														sel_ph3        <= '0';
														if (bdi_decrypt = '1')then
																init_enc_dec <= '0';
																start        <= '1';
																state_next   <= S_TAG_CAL;
														else
																start        <= '1';
																state_next   <= S_TAG_CAL;																					
														end if;
												else
														state_next   <= S_WR;
												end if;	
										end if;
								else                    
										if (bdo_ready = '1')then
												bdo_valid     <= '1';
												sel_ph3        <= '0';
												if (bdi_decrypt = '1')then
													    init_enc_dec <= '0';
													    start        <= '1';
													    state_next   <= S_TAG_CAL;
												else
														start        <= '1';
														state_next   <= S_TAG_CAL;																					
												end if;
										else
												state_next   <= S_WR;
										end if;	
								end if;							  
																					
					end if;					  
					  
                when S_WR =>
					if (bdo_ready = '1')then
							bdo_valid     <= '1';
							sel_ph3        <= '0';
							if (bdi_decrypt = '1')then
									init_enc_dec <= '0';
									start        <= '1';
									state_next   <= S_TAG_CAL;
							else
									start        <= '1';
									state_next   <= S_TAG_CAL;																					
							end if;
					else
							state_next   <= S_WR;
					end if;	
						 
				when S_TAG_CAL =>
						sel_bdo <= '1';
						if (done = '1')then
								if (eot_r = '1')then
										if (decrypt_r = '1')then
										    state_next   <= S_TAG_DEC;
									    else
										    if (bdo_ready = '1')then
                                                bdo_valid    <= '1';
                                                state_next   <= S_RESET;
											else
												state_next   <= S_WR_TAG;
											end if;														
										end if;
							    else
									    state_next   <= S_ENC_DEC;
							    end if; 
                        end if;							 
						 	
				when S_WR_TAG =>
					sel_bdo <= '1';
					if (bdo_ready = '1')then
						bdo_valid    <= '1';
						state_next   <= S_RESET;
					end if;
									  
				when S_TAG_DEC=>
					bdi_ready <= '1';
					if (bdi_valid = '1') then
						if (tag_match = '1')then 
								msg_auth_valid  <= '1';
								msg_auth_done   <= '1';
								state_next  	<= S_RESET;
						else
								msg_auth_done   <= '1';
								state_next   	<= S_RESET;
						end if;
					end if;
                    
	    end case;
	end process;
				
end behavioral;                               


